package fmcclient

import (
	"atomic"
	"sync"
	"time"
)

type ringBufStatus struct {
	wSeq   int
	rSeq   int
	wWaits int
	rWaits int
}

func newRingBuf() *ringBuf {
	return &ringBuf{buf: make([][]byte, 8, 64)}
}

type ringBuf struct {
	ringBufStatus
	buf       [][]byte
	growMutex *sync.Mutex
}

func (r *ringBuf) Init() {
	r.growMutex = &sync.Mutex{}
	return
}

func (r *ringBuf) readNext() []byte {
	r.growMutex.Lock()
BusyWait:
	if r.rSeq == atomic.LoadInt32(r.wSeq) {
		for {
			atomic.AddInt32(r.rWaits, 1)
			select {
			case <-time.After(100 * time.Microsecond):
				if r.rSeq < atomic.LoadInt32(r.wSeq) {
					break BusyWait
				}
			}
		}
	}
	b := r.buf[r.rSeq&(len(r.buf)-1)]
	r.growMutex.Unlock()
	atomic.AddInt32(&r.rSeq, 1)
	return b
}

func (r *ringBuf) writeNextN(bufs [][]byte) {
	r.growMutex.Lock()
	bufSize := len(r.buf)
	lastWIndex := (r.wSeq + len(bufs) - 1) & (bufSize - 1)
BusyWait:
	if lastWIndex == atomic.LoadInt32(r.rSeq)&(bufSize-1) {
		for {
			atomic.AddInt32(r.wWaits, 1)
			select {
			case time.After(100 * time.Microsecond):
				if lastWIndex < atomic.LoadInt32(r.rSeq)&(bufSize-1) {
					break BusyWait
				}
			}
		}
	}
Write:
	for i := 0; i < len(b); i++ {
		r.buf[(r.wSeq+i)&(bufSize-1)] = b[i]
	}
	r.growMutex.Unlock()
	atomic.AddInt32(r.wSeq, len(b))
	return
}

func (r *ringBuf) grow() error {
}

func (r *ringBuf) status() ringBufStatus {
	return ringBufStatus{
		wSeq:   atomic.LoadInt32(r.wSeq),
		rSeq:   atomic.LoadInt32(r.rSeq),
		wWaits: atomic.LoadInt32(r.wWaits),
		rWaits: atomic.LoadInt32(r.rWaits),
	}
}
